@@ -240,12 +240,12 @@ class Cell:
         if prev_cell_type != "markdown":
             self.notebook.dirty = True
             self.json["cell_type"] = "markdown"
-            if "outputs" in self.json:
-                del self.json["outputs"]
-            if "execution_count" in self.json:
-                del self.json["execution_count"]
+            #if "outputs" in self.json:
+            #    del self.json["outputs"]
+            #if "execution_count" in self.json:
+            #    del self.json["execution_count"]
             #self.input_prefix.content = FormattedTextControl(text="")
-            self.input_prefix.content = ""
+            #self.input_prefix.content = ""
             self.clear_output()
             self.set_input_readonly()
             #if prev_cell_type == "code":
@@ -258,12 +258,12 @@ class Cell:
         if prev_cell_type != "code":
             self.notebook.dirty = True
             self.json["cell_type"] = "code"
-            self.json["outputs"] = []
-            self.json["execution_count"] = None
+            #self.json["outputs"] = []
+            #self.json["execution_count"] = None
             #self.c.scr.addstr(1,0, "In [ ]:")
             #text = self.rich_print("\nIn [ ]:" + self.fold_tag(), style="green")
             #self.input_prefix.content = FormattedTextControl(text=text)
-            self.input_prefix.content = text
+            #self.input_prefix.content = text
             self.set_input_readonly()
             if prev_cell_type == "markdown":
                 #self.input = Frame(self.input_window, style="green")
