@@ -162,12 +162,12 @@ class Notebook(Help, Format):
           self.yscr = 0
           self.xscr = 0
           self.lscr = 0
-          self.update_layout()
           self.focus(0)
           self.gmod=0
           #self.output_wrap=False
           self.output_wrap=True
           self.lno=True
+          self.update_layout()
 
     def set_language(self):
         self.kernel_name = self.json["metadata"]["kernelspec"]["name"]
@@ -189,6 +189,67 @@ class Notebook(Help, Format):
           except RuntimeError:
             self.kd = None
  
+
+    def display_help(self):
+      log("[?] notebook.display_help")
+      help_txt="""----------------------------------------------
+
+           CONSOLE JUPYTER NOTEBOOK
+
+  help - press j,k,q (down, up, exit help)
+
+----------------------------------------------
+
+  Keyboard:
+
+  enter, ctrl+e - run current cell
+  t - run as external program (e.g. curses)
+  u, page up - scroll fullscreen page up
+  n, page down - scroll fulscreen page down
+  h, left - scroll fullscreen left more
+  l, right - scroll fullscreen right more
+  h, left - scroll fullscreen left
+  m - toggle markdown cell
+  l, right - scroll fullscreen right
+  i - switch to input
+  o - switch to output
+  w - toggle wrap output
+  f - switch current pad full-screen
+  r - reset layout
+  gg - go to first cell
+  shift+g (G) - go to last cell
+  k, up - previous cell
+  j, down - next cell
+  a - insert cell above current cell
+  a - insert cell below current cell
+  x - cut cell
+  c - copy the cell
+  ctrl+v - paste cell above
+  ctrl+v - paste cell below
+  s, ctrl+s - save notebook
+  e - edit input cell [1]
+  ctrl+p - run all cells one by one
+  q - quit
+
+----------------------------------------------"""
+      self.clear_pads(False)
+      self.c.scr.timeout(-1)
+      k="0"
+      ys=0
+      while k!=ord("q"):
+        self.c.pad_prn(POU,0,0,help_txt)
+        if k==ord("j"):
+          if (len(help_txt.split("\n"))-ys>1):
+            ys+=1
+        elif k==ord("k"):
+          if ys>0:
+            ys-=1
+        self.c.pad_refresh(POU,ys,0,0,0,self.c.he-1,self.c.wi-1)
+        k = self.c.scr.getch()
+        self.c.get_term_size()
+      self.c.scr.timeout(REFRESH_INTERVAL)
+      self.valid = 0
+
     async def kernel_stop(self):
       await self.kd.stop()
 
@@ -284,7 +345,7 @@ class Notebook(Help, Format):
     def print_cell_execution_count(self):
       log("[*] console.print_cell_execution count")
       #log(str(self.current_cell.json))
-      if "execution_count" in self.current_cell.json:
+      if "execution_count" in self.current_cell.json and self.current_cell.json["cell_type"]!="markdown":
           execution_count = self.current_cell.json["execution_count"]
           if execution_count:
             #shex=len(str(execution_count))
@@ -311,7 +372,7 @@ class Notebook(Help, Format):
       #self.print_cell_execution_count()
       #if len(text)==0:
       #self.c.pads[POU].clear()
-      text=""
+      #text=""
       error_msg=0
       try:
         if "outputs" in self.current_cell.json:
@@ -325,8 +386,8 @@ class Notebook(Help, Format):
                 else:
                   text, height = self.current_cell.get_output_text(outputs)
       except:
-        pass
-      #log("[.] notebook.print_cell_output: "+str(text))
+        #pass
+        log("[!] notebook.print_cell_output: "+str(text))
       if error_msg==1:
         log("[.] notebook.print_cell_output: "+str(outputs))
         f=open("traceback.tmp","bw")
@@ -496,14 +557,16 @@ class Notebook(Help, Format):
     def switch_to_input(self):
         if self.pad_active=="O":
             self.pad_active="I"
-            #self.layout_reset()
+            self.layout_reset()
             self.valid=0
+            #self.current_cell.update_json()
 
     def switch_to_output(self):
         if self.pad_active=="I":
             self.pad_active="O"
-            #self.layout_reset()
+            self.layout_reset()
             self.valid=0
+            #self.current_cell.update_json()
 
     def fullscreen_scroll_up(self):
         if self.is_fullscreen()==1:
@@ -624,6 +687,8 @@ class Notebook(Help, Format):
         #self.c.scr.timeout(REFRESH_INTERVAL)
         self.keymap = Keymap()
         self.keymapstack = KeymapStack()
+        #KS 1 - display help 
+        self.keymap.bind("1", self.display_help, ())
         #KS enter, ctrl+e - run current cell
         self.keymap.bind([5,13,'\n', curses.KEY_ENTER], self.run_current_cell, ())
         #KS t - run as external program (e.g. curses)
@@ -707,6 +772,7 @@ class Notebook(Help, Format):
             self.gmod=0
           self.keymapstack.process(k)
           if self.quitting:
+            self.c.end_curses()
             exit(0)
           await sleep(0.05)
 
@@ -717,16 +783,10 @@ class Notebook(Help, Format):
 
     def update_layout(self):
         try:
-          # PRINT PADS
+          # CLEAR AND PRINT PADS
           if self.valid == 0:
-            self.c.scr.erase()
-            self.c.scr.refresh()
             self.clear_pads()
-            #self.c.scr.clear()
-            #self.c.scr.erase()
             self.status_line_valid = 0
-            #self.c.scr.clear()
-            #self.clear_pads()
             self.print_cell_input()
             self.print_cell_output()
             self.print_cell_execution_count()
@@ -741,7 +801,9 @@ class Notebook(Help, Format):
             self.status_line_valid = 1
           self.c.scr.refresh()
         except Exception as e:
+          self.clear_pads()
           log("[!] update_layout exception "+str(e))
+          self.c.scr.getch()
           pass
 
     def create_layout(self):
@@ -760,21 +822,9 @@ class Notebook(Help, Format):
               )
           )
         except Exception as e:
+          self.clear_pads()
           log("-- CREATE LAYOUR PROBLEM: "+str(e))
-        #nb_window = ScrollablePane(HSplit(inout_cells), show_scrollbar=False)
-        #nb_window.preferred_width(40)
-
-    #    def get_top_bar_text():
-    #        text = ""
-    #        if self.dirty:
-    #            text += "+ "
-    #        text += str(self.nb_path.relative_to(self.kernel_cwd))
-    #        if self.dirty and self.quitting:
-    #            text += (
-    #                " (no write since last change, please exit again to confirm, "
-    #                "or save your changes)"
-    #            )
-    #        return text
+          self.c.scr.getch()
 
     # HERE BOTTOM BAR TEXT
     def get_bottom_bar_text(self,exe_state="idle"):
@@ -816,14 +866,6 @@ class Notebook(Help, Format):
         text += f"[{self.current_cell_idx+1}/{len(self.cells)}]"
         return(text)
 
-    #    self.top_bar = FormattedTextToolbar(get_top_bar_text, style="Green bg:Black")
-    #    self.bottom_bar = FormattedTextToolbar(
-    #        get_bottom_bar_text, style="Green bg:Black"
-    #    )
-    #    root_container = HSplit([self.top_bar, nb_window, self.bottom_bar])
-    #    self.layout = Layout(root_container)
-
-    #def focus(self, idx: int, update_layout: bool = False, no_change: bool = False):
     def focus(self, idx: int):
         if idx >= 0 and idx < len(self.cells):
           ##if self.app:
@@ -836,19 +878,6 @@ class Notebook(Help, Format):
           #  self.update_layout()
           #self.c.scr.refresh()
 
-    #def focus_current_cell(self):
-    #    idx = self.current_cell_idx
-    #    self.app = cast(Application, self.app)
-    #    size = self.app.renderer.output.get_size()
-    #    available_height = size.rows - 2  # status bars
-    #    # focus current cell
-    #    (
-    #        self.top_cell_idx,
-    #        self.bottom_cell_idx,
-    #    ) = self.get_visible_cell_idx_from_top(idx, available_height)
-    #    self.focus(idx, update_layout=True)
-    #    return True
-
     def update_visible_cells(self, idx: int, no_change: bool) -> bool:
         #self.app = cast(Application, self.app)
         #size = self.app.renderer.output.get_size()
@@ -959,27 +988,22 @@ class Notebook(Help, Format):
 
     def clear_all_output(self):
         for i in range(0, len(self.cells)):
-            self.focus(i)
-            self.current_cell.clear_output()
+          self.focus(i)
+          self.current_cell.clear_output()
         self.focus(0)
 
     def clear_output(self):
         self.current_cell.clear_output()
 
     def toggle_markdown_cell(self):
-        #if self.current_cell.json["cell_type"] == "markdown":
-        #        self.current_cell.set_as_code()
-        #else:
-        #        self.current_cell.set_as_markdown()
-        #self.notebook.dirty = True
-        #self.current_cell.json["cell_type"] = "markdown"
-        #log("[*] "+str(self.current_cell.json["cell_type"]))
+        text=""
         if self.current_cell.json["cell_type"]=="code":
-          self.current_cell.json["cell_type"]="markdown"
+          self.markdown_cell()
+          self.switch_to_input()
         elif self.current_cell.json["cell_type"]=="markdown":
-          self.current_cell.json["cell_type"]="code"
-        self.dirty = True
-        self.valid = 0
+          self.code_cell()
+          self.switch_to_output()
+        self.update_layout()
 
     def markdown_cell(self):
         self.current_cell.set_as_markdown()
@@ -1161,10 +1185,10 @@ class Notebook(Help, Format):
       except Exception as e:
         print("Kernel stop error." + str(e))
         log("Kernel stop error." + str(e))
-      self.c.end_curses()
       log("[x] conjuno exit ...")
-      print("[x] conjuno exit ...")
+      #print("[x] conjuno exit ...")
       self.quitting=True 
+      self.c.end_curses()
       #exit(0)
 
     def go_up(self):
